/*
 * Proof-of-concept driver for Samsung SWC-U200 wimax dongle.
 * This file contains binary protocol realization.
 * Copyright (C) 2008-2009 Alexander Gordeev <lasaine@lvk.cs.msu.su>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <string.h>

//#include "logging.h"
#include "protocol.h"


int fill_protocol_info_req(unsigned char *buf, int len, unsigned char flags)
{
	buf[0x00] = 0x57;
	buf[0x01] = 0x45;
	buf[0x02] = 0x04;
	buf[0x03] = 0x00;
	buf[0x04] = 0x00;
	buf[0x05] = 0x02;
	buf[0x06] = 0x00;
	buf[0x07] = flags;
	return 8;
}


static void fill_C_req(unsigned char *buf, int len)
{
	len -= 6;
	buf[0x00] = 0x57;
	buf[0x01] = 0x43;
	buf[0x02] = len & 0xff;
	buf[0x03] = (len >> 8) & 0xff;
	memset(buf + 6, 0, 12);
}

static int fill_normal_C_req(unsigned char *buf, unsigned short type_a, unsigned short type_b, unsigned short param_len, unsigned char *param)
{
	int len = 0x1a + param_len;
	fill_C_req(buf, len);
	buf[0x04] = 0x15;
	buf[0x05] = 0x00;
	buf[0x12] = 0x15;
	buf[0x13] = 0x00;
	buf[0x14] = type_a >> 8;
	buf[0x15] = type_a & 0xff;
	buf[0x16] = type_b >> 8;
	buf[0x17] = type_b & 0xff;
	buf[0x18] = param_len >> 8;
	buf[0x19] = param_len & 0xff;
	memcpy(buf + 0x1a, param, param_len);
	return len;
}

int fill_string_info_req(unsigned char *buf, int len)
{
	return fill_normal_C_req(buf, 0x8, 0x1, 0x0, NULL);
}

int fill_diode_control_cmd(unsigned char *buf, int len, int turn_on)
{
	unsigned char param[0x2] = {0x0, turn_on ? 0x1 : 0x0};
	return fill_normal_C_req(buf, 0x30, 0x1, sizeof(param), param);
}

int fill_mac_req(unsigned char *buf, int len)
{
	return fill_normal_C_req(buf, 0x3, 0x1, 0x0, NULL);
}

int fill_auth_policy_req(unsigned char *buf, int len)
{
	return fill_normal_C_req(buf, 0x20, 0x8, 0x0, NULL);
}

int fill_auth_method_req(unsigned char *buf, int len)
{
	return fill_normal_C_req(buf, 0x20, 0xc, 0x0, NULL);
}

int fill_auth_set_cmd(unsigned char *buf, int len)
{
	unsigned char param[0xd] = {0x00, 0x10, 0x00, 0x09, 0x40, 0x79, 0x6f, 0x74, 0x61, 0x2e, 0x72, 0x75, 0x00};
	return fill_normal_C_req(buf, 0x20, 0x20, sizeof(param), param);
}

int fill_find_network_req(unsigned char *buf, int len, unsigned short level)
{
	unsigned char param[0x2] = {level >> 8, level & 0xff};
	return fill_normal_C_req(buf, 0x1, 0x1, sizeof(param), param);
}

int fill_connection_params1_req(unsigned char *buf, int len)
{
	unsigned char param[0x2] = {0xff, 0xff};
	return fill_normal_C_req(buf, 0x1, 0x9, sizeof(param), param);
}

int fill_connection_params2_req(unsigned char *buf, int len)
{
	unsigned char param[0x2] = {0x00, 0x00};
	return fill_normal_C_req(buf, 0x1, 0x9, sizeof(param), param);
}

int fill_state_req(unsigned char *buf, int len)
{
	return fill_normal_C_req(buf, 0x1, 0xb, 0x0, NULL);
}

int fill_network_list_req(unsigned char *buf, int len)
{
	unsigned char param[0x2] = {0x00, 0x00};
	return fill_normal_C_req(buf, 0x24, 0x1, sizeof(param), param);
}


int get_D_header_len()
{
	return 6;
}

int fill_outgoing_packet_header(unsigned char *buf, int len, int body_len)
{
	buf[0x00] = 0x57;
	buf[0x01] = 0x44;
	buf[0x02] = body_len & 0xff;
	buf[0x03] = (body_len >> 8) & 0xff;
	return body_len + 6;
}

